@@ -1,9 +1,10 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-__version__ = "6.0.43"
-import gi
+__version__ = "7.0.15"
 import locale
 
+import gi
+
 gi.require_version('Gtk', '3.0')
 gi.require_version('Gdk', '3.0')
 gi.require_foreign('cairo')
@@ -43,3 +44,20 @@ if not hasattr(locale, 'localize'):
                 formatted = locale._strip_padding(formatted, seps)
         return formatted
     setattr(locale, 'localize', localize)
+
+
+def delocalize(string, monetary=False):
+    conv = locale.localeconv()
+
+    # First, get rid of the grouping
+    ts = conv[monetary and 'mon_thousands_sep' or 'thousands_sep']
+    if ts:
+        string = string.replace(ts, '')
+    # next, replace the decimal point with a dot
+    dd = conv[monetary and 'mon_decimal_point' or 'decimal_point']
+    if dd:
+        string = string.replace(dd, '.')
+    return string
+
+
+setattr(locale, 'delocalize', delocalize)
