@@ -1,27 +1,30 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import sys
+import gettext
 import json
 import locale
-import gettext
+import sys
 from functools import wraps
 
 from gi.repository import Gdk, GLib, GObject, Gtk
 from pygtkcompat.generictreemodel import GenericTreeModel
 
-from tryton.config import CONFIG
-from tryton.common.cellrendererbutton import CellRendererButton
-from tryton.gui.window import Window
-from tryton.common.popup_menu import populate
-from tryton.common import RPCExecute, RPCException, node_attributes, Tooltips
-from tryton.common import domain_inversion, simplify, unique_value
-from tryton.pyson import PYSONDecoder
 import tryton.common as common
+from tryton.common import (
+    RPCException, RPCExecute, Tooltips, domain_inversion, node_attributes,
+    simplify, unique_value)
+from tryton.common.cellrendererbutton import CellRendererButton
+from tryton.common.popup_menu import populate, popup
+from tryton.config import CONFIG
+from tryton.gui.window import Window
+from tryton.pyson import PYSONDecoder
+
 from . import View, XMLViewParser
 from .list_gtk.editabletree import EditableTreeView, TreeView
-from .list_gtk.widget import (Affix, Char, Text, Int, Boolean, URL, Date,
-    Time, Float, TimeDelta, Binary, M2O, O2O, O2M, M2M, Selection,
-    MultiSelection, Reference, Dict, ProgressBar, Button, Image)
+from .list_gtk.widget import (
+    M2M, M2O, O2M, O2O, URL, Affix, Binary, Boolean, Button, Char, Date, Dict,
+    Float, Image, Int, MultiSelection, ProgressBar, Reference, Selection, Text,
+    Time, TimeDelta)
 
 _ = gettext.gettext
 
@@ -252,7 +255,6 @@ class AdaptModelGroup(GenericTreeModel):
 class TreeXMLViewParser(XMLViewParser):
 
     WIDGETS = {
-        'biginteger': Int,
         'binary': Binary,
         'boolean': Boolean,
         'callto': URL,
@@ -302,6 +304,7 @@ class TreeXMLViewParser(XMLViewParser):
 
         for affix in node.childNodes:
             affix_attrs = node_attributes(affix)
+            affix_attrs['visual'] = attributes.get('visual')
             if 'name' not in affix_attrs:
                 affix_attrs['name'] = attributes['name']
             if affix.tagName == 'prefix':
@@ -330,18 +333,8 @@ class TreeXMLViewParser(XMLViewParser):
 
         self.view.treeview.append_column(column)
 
-        if 'sum' in attributes:
-            text = attributes['sum'] + _(':')
-            label, sum_ = Gtk.Label(label=text), Gtk.Label()
-
-            hbox = Gtk.HBox()
-            hbox.pack_start(label, expand=True, fill=False, padding=2)
-            hbox.pack_start(sum_, expand=True, fill=False, padding=2)
-            hbox.show_all()
-            self.view.sum_box.pack_start(
-                hbox, expand=False, fill=False, padding=0)
-
-            self.view.sum_widgets.append((attributes['name'], sum_))
+        if 'optional' in attributes:
+            self.view.optionals.append(column)
 
     def _parse_button(self, node, attributes):
         button = Button(self.view, attributes)
@@ -385,13 +378,24 @@ class TreeXMLViewParser(XMLViewParser):
             hbox.pack_start(arrow_widget, expand=False, fill=False, padding=0)
             column.set_clickable(True)
         hbox.show()
-        column.set_widget(hbox)
+        if bool(int(attributes.get('sum', 0))):
+            vbox = Gtk.VBox(homogeneous=False, spacing=0)
+            vbox.pack_start(hbox, expand=False, fill=True, padding=0)
+            sum_ = Gtk.Label()
+            sum_.set_justify(Gtk.Justification.RIGHT)
+            sum_.show()
+            vbox.pack_start(sum_, expand=False, fill=True, padding=0)
+            self.view.sum_widgets.append((attributes['name'], sum_))
+            vbox.show()
+            widget = vbox
+        else:
+            widget = hbox
+        column.set_widget(widget)
         column.set_alignment(align)
 
     def _set_column_width(self, column, attributes):
         default_width = {
             'integer': 80,
-            'biginteger': 80,
             'selection': 90,
             'reference': 200,
             'one2many': 50,
@@ -431,10 +435,12 @@ class ViewTree(View):
 
     def __init__(self, view_id, screen, xml, children_field):
         self.children_field = children_field
+        self.optionals = []
         self.sum_widgets = []
         self.sum_box = Gtk.HBox()
         self.treeview = None
         self._editable = bool(int(xml.getAttribute('editable') or 0))
+        self._creatable = bool(int(xml.getAttribute('creatable') or 1))
         if self._editable:
             self.treeview = EditableTreeView(self)
             grid_lines = Gtk.TreeViewGridLines.BOTH
@@ -447,17 +453,6 @@ class ViewTree(View):
 
         self.mnemonic_widget = self.treeview
 
-        # Add last column if necessary
-        for column in self.treeview.get_columns():
-            if column.get_expand():
-                break
-        else:
-            column = Gtk.TreeViewColumn()
-            column._type = 'fill'
-            column.name = None
-            column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
-            self.treeview.append_column(column)
-
         self.treeview.set_property('enable-grid-lines', grid_lines)
         self.treeview.set_fixed_height_mode(
             all(c.get_sizing() == Gtk.TreeViewColumnSizing.FIXED
@@ -467,6 +462,8 @@ class ViewTree(View):
         self.treeview.connect_after('row-activated', self.__sig_switch)
         if self.children_field:
             self.treeview.connect('test-expand-row', self.test_expand_row)
+            self.treeview.connect('row-expanded', self._row_expanded)
+            self.treeview.connect('row-collapsed', self._row_collapsed)
         self.treeview.set_rubber_banding(True)
 
         selection = self.treeview.get_selection()
@@ -491,6 +488,58 @@ class ViewTree(View):
 
         self.display()
 
+        if self.optionals:
+            if self.draggable:
+                column = self.treeview.get_columns()[0]
+            else:
+                column = Gtk.TreeViewColumn()
+                column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
+                column.name = None
+                column._type = 'optional'
+                self.treeview.insert_column(column, 0)
+            image = Gtk.Image()
+            image.set_from_pixbuf(common.IconFactory.get_pixbuf('tryton-menu'))
+            image.show()
+            column.set_widget(image)
+            column.set_fixed_width(25)
+            column.set_clickable(True)
+            column.connect('clicked', self.optional_menu)
+
+        # Add last column if necessary after display for updated visible
+        for column in self.treeview.get_columns():
+            if column.get_expand() and column.get_visible():
+                break
+        else:
+            column = Gtk.TreeViewColumn()
+            column._type = 'fill'
+            column.name = None
+            column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
+            self.treeview.append_column(column)
+
+    def optional_menu(self, column):
+        def toggle(menuitem, column):
+            column.set_visible(menuitem.get_active())
+            self.save_optional()
+
+        widget = column.get_widget()
+        menu = Gtk.Menu()
+        for optional in self.optionals:
+            menuitem = Gtk.CheckMenuItem(label=optional.get_title())
+            menuitem.set_active(optional.get_visible())
+            menuitem.connect('toggled', toggle, optional)
+            menu.add(menuitem)
+        popup(menu, widget)
+
+    def save_optional(self):
+        fields = {c.name: not c.get_visible() for c in self.optionals}
+        try:
+            RPCExecute(
+                'model', 'ir.ui.view_tree_optional', 'set_optional',
+                self.view_id, fields)
+        except RPCException:
+            pass
+        self.screen.tree_column_optional[self.view_id] = fields
+
     def get_column_widget(self, column):
         'Return the widget of the column'
         idx = [c for c in self.treeview.get_columns()
@@ -623,6 +672,10 @@ class ViewTree(View):
     def editable(self):
         return self._editable and not self.screen.readonly
 
+    @property
+    def creatable(self):
+        return self.editable and self._creatable
+
     def get_fields(self, visible_only=False):
         return [col.name for col in self.treeview.get_columns()
             if col.name and (not visible_only or col.get_visible())]
@@ -650,20 +703,22 @@ class ViewTree(View):
             self.record = model.get_value(iter_, 0)
             self.screen.switch_view('form')
             return True
+        return False
+
+    def _row_expanded(self, treeview, iter_, path):
+        # Force record_message
+        self.screen.current_record = self.screen.current_record
+
+    def _row_collapsed(self, treeview, iter_, path):
+        # Force record_message
+        self.screen.current_record = self.screen.current_record
+        model = treeview.get_model()
         iter_ = model.iter_children(iter_)
-        if not iter_:
-            return False
-        fields = [col.name for col in self.treeview.get_columns()
-                if col.name]
         while iter_:
             record = model.get_value(iter_, 0)
-            if not record.get_loaded(fields):
-                for field in fields:
-                    record[field]
-                    if record.exception:
-                        return True
+            if record.exception:
+                record.cancel()
             iter_ = model.iter_next(iter_)
-        return False
 
     def on_copy(self):
         for clipboard_type in [
@@ -905,14 +960,16 @@ class ViewTree(View):
             return True
         return False
 
-    def group_list_changed(self, group, signal):
+    def group_list_changed(self, group, action, *args):
         model = self.treeview.get_model()
         if model is not None:
-            if signal[0] == 'record-added':
-                model.added(group, signal[1])
-            elif signal[0] == 'record-removed':
-                model.removed(group, signal[1])
-        self.display()
+            if action == 'record-added':
+                record, pos = args
+                model.added(group, record)
+            elif action == 'record-removed':
+                record, pos = args
+                model.removed(group, record)
+        self.display(force=action == 'group-cleared')
 
     def __str__(self):
         return 'ViewList (%d)' % id(self)
@@ -1057,25 +1114,34 @@ class ViewTree(View):
             domain.append(tab_domain)
         domain = simplify(domain)
         decoder = PYSONDecoder(self.screen.context)
+        tree_column_optional = self.screen.tree_column_optional.get(
+            self.view_id, {})
         for column in self.treeview.get_columns():
             name = column.name
             if not name:
                 continue
             widget = self.get_column_widget(column)
             widget.set_editable()
-            if decoder.decode(widget.attrs.get('tree_invisible', '0')):
+            if column.name in tree_column_optional:
+                optional = tree_column_optional[column.name]
+            else:
+                optional = bool(int(widget.attrs.get('optional', '0')))
+            invisible = decoder.decode(widget.attrs.get('tree_invisible', '0'))
+            if invisible or optional:
                 column.set_visible(False)
             elif name == self.screen.exclude_field:
                 column.set_visible(False)
-            else:
+            elif name in self.group.fields:
+                field = self.group.fields[name]
                 inv_domain = domain_inversion(domain, name)
                 if not isinstance(inv_domain, bool):
                     inv_domain = simplify(inv_domain)
-                unique, _, _ = unique_value(inv_domain)
+                unique, _, _ = unique_value(
+                    inv_domain, single_value=field._single_value)
                 column.set_visible(not unique or bool(self.children_field))
         if self.children_field:
             for i, column in enumerate(self.treeview.get_columns()):
-                if self.draggable and not i:
+                if (self.draggable or self.optionals) and not i:
                     continue
                 if column.get_visible():
                     self.treeview.set_expander_column(column)
@@ -1136,7 +1202,7 @@ class ViewTree(View):
                         '{}'.format(selected_sum or 0), True)
                     sum_ = locale.localize('{}'.format(sum_ or 0), True)
 
-                text = '%s / %s' % (selected_sum, sum_)
+                text = '%s\n%s' % (selected_sum, sum_)
             else:
                 text = '-'
             label.set_text(text)
@@ -1167,8 +1233,7 @@ class ViewTree(View):
             records.append(model.get_value(iter_, 0))
         records = []
         sel = self.treeview.get_selection()
-        if sel is not None:
-            sel.selected_foreach(_func_sel_get, records)
+        sel.selected_foreach(_func_sel_get, records)
         return records
 
     def get_selected_paths(self):
